/**
 * DCBanner jQuery plugin.
 * @see http://bitbucket.org/sholsinger/dcbanner
 * @author Stephen Holsinger <sholsinger@gmail.com>
 * @copyright 2010-11-11
 */
(function($) {

/**
 *	Defaults which will be used to fill in missing configuration values.
 */
var defaults = {
	fadeDuration: 500,
	bannerDuration: 10000,
	defaultItem: 0,
	easingFunction: 'linear',
	timer: null
};

/**
 *	Commonly used string keys (helps with minification)
 */
var strings = {
	currentBannerClass: 'active',
	consoleLogPattern: '%s: %o',
	fadingOutClass: 'fading-out',
	fadingInClass: 'fading-in',
	hoverClass: 'hover',
	initializedClass: 'dcbanner',
	optKey: 'dcbanner-options',
	autoAdvanceFunction: 'dcbanner-autoadvancefn'
};

/**
 * Logging function to allow chaining logging onto a jQuery call.
 */
$.fn.log = function ( msg )
{
	log( msg, this );
	return this;
};

/**
 * Logging function to enable logging to JS console.
 */
function log ( message, element )
{
	return;
	if( $.browser.msie ){/*alert(message+": "+element);*/ return false;}
	if( window.console && window.console.log )
	{
		console.log( strings.consoleLogPattern, message, element );
	}
}

/**
 * Define the plugin (pre-initialization stuff)
 */
$.fn.dcbanner = function( args )
{
	// merge defaults with given opts.
	args = $.extend( {}, defaults, args );
	
	// allow the use of the meta-data plugin
	args = $.meta ? $.extend( {}, args, $this.data() ) : args;
	
	// iterate over matched elements
	return this.each( function()
	{
		var $this = $(this);
		
		// store settings on the parent element
		saveSettings( args, $this );
		
		// mark this as being initialized
		$this.addClass( strings.initializedClass );
		
		// initialize the plugin on the matched element.
		init( $this );
	});
};

/**
 * Initialize the plugin onto the element passed in $el
 */
function init( $el )
{
	var opts = getSettings( $el );
	$('a', $el).click(navClickHandler);
	$('img', $el).click(bannerClickHandler).fadeOut(0);
	$('li', $el)
		.removeClass( strings.currentBannerClass )
		.eq( opts.defaultItem )
			.addClass( strings.currentBannerClass )
			.find('img')
			.fadeIn( opts.fadeDuration );
	$el.bind( strings.autoAdvanceFunction, autoAdvance );
	opts.timer = setTimeout(
		function(){$el.trigger( strings.autoAdvanceFunction );}, 
		opts.bannerDuration
	);
}

/**
 * click handler function for the "tabs"
 */
function navClickHandler( e )
{
	e.preventDefault();
	var $li = $(e.target).closest('li');
	
	// fast fail if the element clicked is already active.
	if($li.is( className(strings.currentBannerClass) ))
	{
		return false;
	}
	
	$banner = getBanner(e.target);
	
	$banner.find('img').stop(true, true);
	
	var opts = getSettings( $banner );
	
	clearTimeout( opts.timer );
	
	doBannerFade( $li, $li.siblings( className(strings.currentBannerClass) ).eq(0) );
	
	// restart the timeout when the user moves mouse off of the banner.
	$banner.one('mouseout', function(e)
		{
			log("Restarting timer on ", this);
			var opts = getSettings( this );
			var $this = $(this);
			opts.timer = setTimeout( function(){ $this.trigger( strings.autoAdvanceFunction ); }, opts.bannerDuration );
			saveSettings( opts, this );
		});
}

/**
 * click handler function for the "banners"
 */
function bannerClickHandler( e )
{
	e.preventDefault();
	window.location = $(e.target).parent('a').attr('href');
}

/**
 * Execute cross-fade between $new and $old elements
 */
function doBannerFade( $new, $old )
{
	log('doBannerFade(): args = ', {'new': $new.eq(0), 'old': $old.eq(0)} );
	$new.addClass( strings.fadingInClass + ' ' + strings.currentBannerClass );
	$old.addClass( strings.fadingOutClass )
		.removeClass( strings.currentBannerClass );
	
	$new.find('img').fadeIn( opts.fadeDuration, opts.easingFunction, fadeInCleanup ).log('fading in');
	$old.find('img').fadeOut( opts.fadeDuration, opts.easingFunction, fadeOutCleanup ).log('fading out');
}

// clean up after the fade operation.
function fadeInCleanup()
{
	log('fadeInCleanup(): ', this);
	$(this).closest('li').removeClass( strings.fadingInClass );
}

// clean up after the fade operation.
function fadeOutCleanup()
{
	log('fadeOutCleanup(): ', this);
	$(this).closest('li').removeClass( strings.fadingOutClass );
}

// automatically advance to the next element in the banner.
function autoAdvance()
{
	log("autoAdvance() called on", this);
	var $this = $(this);
	var opts = getSettings(this);
	clearTimeout( opts.timer );
	$this.stop(true, true);
	var $current = $this.find( className(strings.currentBannerClass) );
	var $next = $this.find( className(strings.currentBannerClass) +' ~ li').eq(0);
	if($next.length < 1)
	{
		$next = $this.find('li').eq(0);
	}
	doBannerFade( $next, $current );
	opts.timer = setTimeout(
		function(){ $this.trigger( strings.autoAdvanceFunction ); },
		opts.bannerDuration
	);
	saveSettings( opts, this );
}

// append a period to a classname
function className(str)
{
	return '.'+str;
}

// get the "banner" for the given element.
function getBanner( child )
{
	return $(child).closest( className(strings.initializedClass) );
}

// save the settings passed in opts to the given element.
function saveSettings( opts, el )
{
	return $(el).data( strings.optKey, opts );
}

// get the settings for the banner that the given element belongs to.
function getSettings( el )
{
	return opts = getBanner(el).data( strings.optKey );
}
	
})(jQuery);